@@ -1,21 +1,21 @@
-#
+
 #    FormValidator.pm - Object that validates form input data.
 #
 #    This file is part of Data::FormValidator.
 #
-#    Author: Francis J. Lacoste <francis.lacoste@iNsu.COM>
+#    Author: Francis J. Lacoste 
 #    Maintainer: Mark Stosberg <mark@stosberg.com>
 #
 #    Copyright (C) 1999 Francis J. Lacoste, iNsu Innovations
-#    Parts Copyright 1996-1999 by Michael J. Heins <mike@heins.net>
-#    Parts Copyright 1996-1999 by Bruce Albrecht  <bruce.albrecht@seag.fingerhut.com>
-#    Parts Copyright 2001-2003 by Mark Stosberg <mark@stosberg.com>
+#    Parts Copyright 1996-1999 by Michael J. Heins 
+#    Parts Copyright 1996-1999 by Bruce Albrecht  
+#    Parts Copyright 2001-2005 by Mark Stosberg 
 #
 #    Parts of this module are based on work by
-#    Bruce Albrecht, <bruce.albrecht@seag.fingerhut.com> contributed to
+#    Bruce Albrecht,  contributed to
 #    MiniVend.
 #
-#    Parts also based on work by Michael J. Heins <mikeh@minivend.com>
+#    Parts also based on work by Michael J. Heins 
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms same terms as perl itself.
@@ -23,7 +23,7 @@
 
 package Data::FormValidator;
 
-use 5.005; # for "qr" support, which isn't strictly required. -mls
+use 5.005; # for "qr" support, which isn't strictly required. 
 
 use Data::FormValidator::Results;
 use Data::FormValidator::Filters (qw/:filters/);
@@ -31,7 +31,7 @@ use Data::FormValidator::Constraints (qw/:validators :matchers/);
 
 use vars qw( $VERSION $AUTOLOAD @ISA @EXPORT_OK %EXPORT_TAGS );
 
-$VERSION = '3.63';
+$VERSION = '4.40';
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -161,17 +161,15 @@ sub new {
     }, $class;
 }
 
-=pod
-
 =head1 VALIDATING INPUT
 
 =head2 check()
 
  my $results = Data::FormValidator->check(\%input_hash, \%dfv_profile);
 
-C<check> is the recommended method to use to validate forms. It returns it's results as a
+C<check> is the recommended method to use to validate forms. It returns its results as a
 L<Data::FormValidator::Results|Data::FormValidator::Results> object.  A
-deprecated method C<validate> is also available, returning it's results as an
+deprecated method C<validate> is also available, returning its results as an
 array described below.
 
  use Data::FormValidator;
@@ -211,8 +209,9 @@ I<dependencies> list.
 
 =item invalids
 
-This is a reference to an array which contains the name of the fields
-which failed one or more of their constraint checks.
+This is a reference to an array which contains the name of the fields which
+failed one or more of their constraint checks. If there are no invalid fields,
+an empty arrayref will be returned. 
 
 Fields defined with multiple constraints will have an array ref returned in the
 @invalids array instead of a string. The first element in this array is the
@@ -229,10 +228,7 @@ dependant.
 
 =head2 new()
 
-Using C<new()> is only needed for advanced usage, so feel free to skip this
-section if you are just getting started.
-
-That said, using C<new()> is useful in some cases. These include:
+Using C<new()> is only needed for advanced usage, including these cases:
 
 =over
 
@@ -243,7 +239,7 @@ want by name later with C<check()>. Here's an example:
 
  my $dfv = Data::FormValidator->new({
     profile_1 => { # usual profile definition here },
-    profile_1 => { # another profile definition },
+    profile_2 => { # another profile definition },
  });
 
 
@@ -268,7 +264,7 @@ Applying defaults to more than one input profile. There are some parts
 of the validation profile that you might like to re-use for many form
 validations. 
 
-To facilite this, C<new()> takes a second argument, a hash reference. Here
+To facilitate this, C<new()> takes a second argument, a hash reference. Here
 the usual input profile definitions can be made. These will act as defaults for
 any subsequent calls to C<check()> on this object.
 
@@ -315,7 +311,7 @@ sub validate {
 
     my $valid   = $data_set->valid();
     my $missing = $data_set->missing();
-    my $invalid = $data_set->{validate_invalid};
+    my $invalid = $data_set->{validate_invalid} || [];
     my $unknown = [ $data_set->unknown ];
 
     return ( $valid, $missing, $invalid, $unknown );
@@ -357,9 +353,6 @@ sub check {
     return $results;
 }
 
-
-=pod
-
 =head1 INPUT PROFILE SPECIFICATION
 
 An input profile is a hash reference containing one or more of the following
@@ -368,6 +361,8 @@ keys.
 Here is a very simple input profile. Examples of more advanced options are
 described below.
 
+    use Data::FormValidator::Constraints qw(:closures);
+
     my $profile = {
         optional => [qw( company
                          fax 
@@ -378,8 +373,8 @@ described below.
                          email 
                          address )],
 
-        constraints => {
-            email => 'email'
+        constraint_methods => {
+            email => email(), 
         }
     };
 
@@ -400,8 +395,8 @@ spaces will be reported as missing.
 
  required_regexp => qr/city|state|zipcode/,
 
-This is a regular expression used to specify additional fields which are
-required.
+This is a regular expression used to specify additional field names for which values
+will be required.
 
 =head2 require_some
 
@@ -437,7 +432,6 @@ to be optional, you could use the regular expression, /^user_/
 
 =head2 dependencies
 
-
  dependencies   => {
 
     # If cc_no is entered, make cc_type and cc_exp required
@@ -470,7 +464,6 @@ This is a hash reference which contains information about groups of
 interdependent fields. The keys are arbitrary names that you create and
 the values are references to arrays of the field names in each group. 
 
-
 =head2 defaults
 
  defaults => {
@@ -480,25 +473,28 @@ the values are references to arrays of the field names in each group.
 This is a hash reference where keys are field names and 
 values are defaults to use if input for the field is missing. 
 
+The values can be code refs which will be used to calculate the 
+value if needed. These code refs will be passed in the DFV::Results
+object as the only parameter. 
+
 The defaults are set shortly before the constraints are applied, and
 will be returned with the other valid data.
 
-
 =head2 filters
 
  # trim leading and trailing whitespace on all fields
  filters       => ['trim'],
 
-This is a reference to an array of filters that will be applied to ALL
-optional and required fields. 
+This is a reference to an array of filters that will be applied to ALL optional
+and required fields, B<before> any constraints are applied. 
 
 This can be the name of a built-in filter
 (trim,digit,etc) or an anonymous subroutine which should take one parameter, 
 the field value and return the (possibly) modified value.
 
-Filters modify the data, so use them carefully. 
+Filters modify the data returned through the results object, so use them carefully. 
 
-See Data::FormValidator::Filters for details on the built-in filters.
+See L<Data::FormValidator::Filters> for details on the built-in filters.
 
 =head2 field_filters
 
@@ -506,10 +502,13 @@ See Data::FormValidator::Filters for details on the built-in filters.
      cc_no => ['digit'],
  },
 
-A hash ref with field names and keys. Values are array references
-of field-specific filters to apply.
+A hash ref with field names as keys. Values are array references of built-in
+filters to apply (trim,digit,etc) or an anonymous subroutine which should take
+one parameter, the field value and return the (possibly) modified value.
 
-See Data::FormValidator::Filters for details on the built-in filters.
+Filters are applied B<before> any constraints are applied. 
+
+See L<Data::FormValidator::Filters> for details on the built-in filters.
 
 =head2 field_filter_regexp_map
 
@@ -523,19 +522,22 @@ use and the values are references to arrays of filters which will be
 applied to specific input fields. Used to apply filters to fields that
 match a regular expression. 
 
-=head2 constraints
+=head2 constraint_methods
 
- constraints => {
-    cc_no      => {  
-        constraint  => "cc_number",
-        params        => [ qw( cc_no cc_type ) ],
-    },
-    cc_type    => "cc_type",
-    cc_exp    => "cc_exp",
+ use Data::FormValidator::Constraints qw(:closures);
+
+ constraint_methods => {
+    cc_no      => cc_number({fields => ['cc_type']}),
+    cc_type    => cc_type(),
+    cc_exp     => cc_exp(),
   },
 
-A hash ref which contains the constraints that
-will be used to check whether or not the field contains valid data.
+A hash ref which contains the constraints that will be used to check whether or
+not the field contains valid data.
+
+B<Note:> To use the built-in constraints, they need to first be loaded into your
+name space using the syntax above. (Unless you are using the old C<constraints> key,  
+documented in L<BACKWARDS COMPATIBILITY>). 
 
 The keys in this hash are field names. The values can be any of the following:
 
@@ -555,14 +557,14 @@ built-in constraints that are available.
 
 =item o 
 
-a perl regular expression
+A perl regular expression
 
 B<Example>: 
 
  my_zipcode_field   => qr/^\d{5}$/, # match exactly 5 digits
 
-If this field is named in the C<untaint_constraint_fields>, or 
-C<untaint_all_constraints> is effective, be aware of the following: If you
+If this field is named in C<untaint_constraint_fields> or C<untaint_regexp_map>,
+or C<untaint_all_constraints> is effective, be aware of the following: If you
 write your own regular expressions and only match part of the string then
 you'll only get part of the string in the valid hash. It is a good idea to
 write you own constraints like /^regex$/. That way you match the whole string.
@@ -572,57 +574,27 @@ write you own constraints like /^regex$/. That way you match the whole string.
 a subroutine reference, to supply custom code
 
 This will check the input and return true or false depending on the input's validity.
-By default, the constraint function takes one parameter, the field to be
-validated.  To validate a field based more inputs than just the field itself,
-see C<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
-
+By default, the constraint function recieves a L<Data::FormValidator::Results>
+object as its first argument, and the value to be validated as the second.  To
+validate a field based more inputs than just the field itself, see 
+L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
 
 B<Examples>:
 
- my_zipcode_field => sub { my $val = shift;  return $val =~ '/^\d{5}$/' }, 
+ # Notice the use of 'pop'-- 
+ # the object is the first arg passed to the method
+ # while the value is the second, and last arg.
+ my_zipcode_field => sub { my $val = pop;  return $val =~ '/^\d{5}$/' }, 
  
  # OR you can reference a subroutine, which should work like the one above
  my_zipcode_field => \&my_validation_routine, 
 
-=item o
-
-a hash reference, to name a constraint or supply multiple parameters.
-
- # supply multiple parameters
- cc_no  => {  
-     constraint  => "cc_number",
-     params         => [ qw( cc_no cc_type ) ],
- },
-
- # name a constraint, useful for returning error messages
- last_name => {
-     name => "ends_in_name",
-     constraint => qr/_name$/,
- },
-
-Using a hash reference for a constraint permits the passing of multiple
-arguments. Required arguments are C<constraint> or C<constraint_method>.
-Optional arguments are C<name> and C<params>.
-
-A C<name> on a constraints 'glues' the constraint to its error message
-in the validator profile (refer C<msgs> section below). If no C<name> is 
-given then it will default to the value of C<constraint> or 
-C<constraint_method> IF they are NOT a CODE ref or a RegExp ref.
-
-The C<params> value is a reference to an array of the parameters to pass 
-to the constraint method. 
-If an element of the C<params> list is a scalar, it is assumed to be naming
-a key of the %input_hash and that value is passed to the routine. 
-If the parameter is a '\' reference, then it is treated literally and passed 
-unchanged to the routine.
-
-If you are using the older C<constraint> over 
-the new C<constraint_method> then don't forget to include the name of the 
-field to check in the C<params> list. C<constraint_method> provides access
-to this value via the C<get_current_*> methods 
-(refer L<Data::FormValidator::Constraints>)
-
-For more details see L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
+ # An example of setting the constraint name.
+ my_zipcode_field => sub { 
+ 	my ($dfv, $val) = @_;
+	$dfv->set_current_constraint_name('my_constraint_name');
+ 	return $val =~ '/^\d{5}$/' 
+	}, 
 
 =item o 
 
@@ -632,21 +604,26 @@ An array reference is used to apply multiple constraints to a single
 field. Any of the above options are valid entries the array.
 See L<MULTIPLE CONSTRAINTS> below.
 
+For more details see L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
+
 =back
 
-=head2 constraint_regexp_map
+=head2 constraint_method_regexp_map
 
- constraint_regexp_map => {
+ use Data::FormValidator::Constraints qw(:closures);
+
+ # In your profile. 
+ constraint_method_regexp_map => {
      # All fields that end in _postcode have the 'postcode' constraint applied.
-     qr/_postcode$/    => 'postcode',
+     qr/_postcode$/    => postcode(),
  },                  
 
 A hash ref where the keys are the regular expressions to
 use and the values are the constraints to apply. 
 
 If one or more constraints have already been defined for a given field using
-"constraints", constraint_regexp_map will add an additional constraint for that
-field for each regular expression that matches.
+C<constraint_methods>, C<constraint_method_regexp_map> will add an additional
+constraint for that field for each regular expression that matches.
 
 =head2 untaint_all_constraints
 
@@ -660,7 +637,7 @@ may not provide untainting.
 See L<WRITING YOUR OWN CONSTRAINT ROUTINES> in the Data::FormValidator::Constraints
 documention for more information.
 
-This is overridden by C<untaint_constraint_fields>
+This is overridden by C<untaint_constraint_fields> and C<untaint_regexp_map>.
 
 =head2 untaint_constraint_fields
 
@@ -672,13 +649,27 @@ field names. The untainted data will be returned in the valid hash.
 
 This overrides the untaint_all_constraints flag.
 
+=head2 untaint_regexp_map
+
+ untaint_regexp_map => [qr/some_field_\d/],
+
+Specifies that certain fields will be untained if they pass their constraints
+and match one of the regular expressions supplied. This can be set to a single
+regex, or an array reference of regexes. The untainted data will be returned
+in the valid hash.
+
+The above example would untaint the fields named C<some_field_1>, and C<some_field_2>
+but not C<some_field>. 
+
+This overrides the untaint_all_constraints flag.
+
 =head2 missing_optional_valid
 
  missing_optional_valid => 1
 
-This can be set to a true value to cause missing optional fields to be included
-in the valid hash. By default they are not included-- this is the historical
-behavior. 
+This can be set to a true value to cause optional fields with empty values to
+be included in the valid hash. By default they are not included-- this is the
+historical behavior. 
 
 This is an important flag if you are using the contents of an "update" form to
 update a record in a database. Without using the option, fields that have been
@@ -701,10 +692,6 @@ documentation for more information
 
 =head2 msgs
 
-B<NOTE:> This part of the interface is newer and may change.  Use in production
-code at your own caution. Contact the maintainer with any questions or
-suggestions.
-
 This key is used to define parameters related to formatting error messages
 returned to the user.
 
@@ -737,9 +724,9 @@ as providing custom messages per field, and handling multiple constraints:
      # Default invalid message, default's to "Invalid"
      invalid => 'Problematic!',
  
-     # message seperator for multiple messages
+     # message separator for multiple messages
      # Defaults to ' '
-     invalid_seperator => ' <br /> ',
+     invalid_separator => ' <br /> ',
  
      # formatting string, default given above.
      format => 'ERROR: %s',
@@ -762,6 +749,24 @@ as providing custom messages per field, and handling multiple constraints:
 The hash that's prepared can be retrieved through the C<msgs> method
 described in the L<Data::FormValidator::Results> documentation.
 
+=head2 msgs - callback 
+
+I<This is a new feature. While it expected to be forward-compatible, it hasn't
+yet received the testing the rest of the API has.>   
+
+If the built-in message generation doesn't suit you, it is also possible to
+provide your own by specifying a code reference:
+
+ msgs  =>  \&my_msgs_callback
+
+This will be called as a L<Data::FormValidator::Results> method.  It may
+receive as arguments an additional hash reference of control parameters,
+corresponding to the key names in the usually used in the C<msgs> area of the
+profile. You can ignore this information if you'd like. 
+
+If you have an alternative error message handler you'd like to share, 
+stick in the C<Data::FormValidator::ErrMsgs> and upload it to CPAN. 
+
 =head2 debug
 
 This method is used to print details about what is going on to STDERR.
@@ -798,17 +803,20 @@ Deprecated, but supported
 
 You can pass more than one value into a constraint routine.  For that, the
 value of the constraint should be a hash reference. If you are creating your
-own routines, be sure to read the section labeled L<WRITING YOUR OWN VALIDATION
-ROUTINES>, in the Data::FormValidator::Constraints documentation.  It describes
+own routines, be sure to read the section labeled 
+L<WRITING YOUR OWN CONSTRAINT ROUTINES>, 
+in the Data::FormValidator::Constraints documentation.  It describes
 a newer and more flexible syntax. 
 
 Using the original syntax, one key should be named C<constraint> and should
 have a value set to the reference of the subroutine or the name of a built-in
 validator.  Another required key is C<params>. The value of the C<params> key
 is a reference to an array of the other elements to use in the validation. If
-the element is a scalar, it is assumed to be a field name. If the value is a
-reference, the reference is passed directly to the routine. Don't forget to
-include the name of the field to check in that list, if you are using this syntax.
+the element is a scalar, it is assumed to be a field name. The field is known
+to Data::FormValidator, the value will be filtered through any defined filteres
+before it is passed in.  If the value is a reference, the reference is passed
+directly to the routine.  Don't forget to include the name of the field to
+check in that list, if you are using this syntax.
 
 B<Example>:
 
@@ -875,6 +883,8 @@ sub _check_profile_syntax {
         die "Invalid input profile: needs to be a hash reference\n";
 
     my %valid_profile_keys = (
+		constraint_methods 	          => undef,
+		constraint_method_regexp_map  => undef,
         constraint_regexp_map=> undef,
         constraints=> undef,
         defaults=> undef,
@@ -893,6 +903,7 @@ sub _check_profile_syntax {
         untaint_all_constraints=> undef,
         validator_packages=> undef,
         untaint_constraint_fields=> undef,
+        untaint_regexp_map => undef,
         debug=> undef,
     );
 
@@ -930,6 +941,26 @@ sub _check_profile_syntax {
         die "Invalid input profile: constraint hashref keys not recognised [@invalid]\n";
     }
 
+	my %valid_msgs_hash_keys = (
+            prefix => undef,
+            missing => undef,
+            invalid => undef,
+            invalid_separator => undef,
+            invalid_seperator => undef,
+            format  => undef,
+            constraints  => undef,
+            any_errors => undef,
+	);
+	if (ref $profile->{msgs} eq 'HASH') {
+		for my $key (keys %{ $profile->{msgs} }) {
+			push @invalid, $key unless exists $valid_msgs_hash_keys{$key};
+		}
+	}
+    if (@invalid) {
+        die "Invalid input profile: msgs keys not recognized: [@invalid]\n";
+    }
+
+
 }
 
 
@@ -943,11 +974,148 @@ __END__
 
 =head1 ADVANCED VALIDATION
 
-For even more advanced validation, you will likely want to read the documentation
-for other modules in this distribution, linked below. Also keep in mind that the  
-Data::FormValidator profile structure is just another data structure. There is
-no reason why it needs to be defined statically. The profile could also be built
-on the fly with custom Perl code.
+For even more advanced validation, you will likely want to read the
+documentation for other modules in this distribution, linked below. Also keep
+in mind that the  Data::FormValidator profile structure is just another data
+structure. There is no reason why it needs to be defined statically. The
+profile could also be built on the fly with custom Perl code.
+
+=head1 BACKWARDS COMPATIBILITY
+
+=head2 validate()
+
+    my( $valids, $missings, $invalids, $unknowns ) = 
+        Data::FormValidator->validate( \%input_hash, \%dfv_profile);
+
+C<validate()> provides a deprecated alternative to C<check()>. It has the same input
+syntax, but returns a four element array, described as follows
+
+=over
+
+=item valids
+
+This is a hash reference to the valid fields which were submitted in
+the data. The data may have been modified by the various filters specified.
+
+=item missings
+
+This is a reference to an array which contains the name of the missing
+fields. Those are the fields that the user forget to fill or filled
+with spaces. These fields may comes from the I<required> list or the
+I<dependencies> list.
+
+=item invalids
+
+This is a reference to an array which contains the name of the fields
+which failed one or more of their constraint checks.
+
+Fields defined with multiple constraints will have an array ref returned in the
+@invalids array instead of a string. The first element in this array is the
+name of the field, and the remaining fields are the names of the failed
+constraints. 
+
+=item unknowns
+
+This is a list of fields which are unknown to the profile. Whether or
+not this indicates an error in the user input is application
+dependant.
+
+=back
+
+=head2 constraints (profile key)
+
+This is a supported but deprecated profile key. Using C<constraint_methods> is
+recommended instead, which provides a simpler, more versatile interface. 
+
+ constraints => {
+    cc_no      => {  
+        constraint  => "cc_number",
+        params        => [ qw( cc_no cc_type ) ],
+    },
+    cc_type    => "cc_type",
+    cc_exp    => "cc_exp",
+  },
+
+A hash ref which contains the constraints that
+will be used to check whether or not the field contains valid data.
+
+The keys in this hash are field names. The values can be any of the following:
+
+=over 
+
+=item o
+
+A named constraint. 
+
+B<Example>: 
+
+ my_zipcode_field     => 'zip',
+
+See L<Data::FormValidator::Constraints> for the details of which
+built-in constraints that are available.
+
+=back
+
+=head2 hashref style of specifying constraints 
+
+Using a hash reference to specify a constraint is an older technique
+used to name a constraint or supply multiple parameters.
+
+Both of these interface issues are now better addressed with C<constraint_methods>
+and C<$self-\>name_this('foo')>.
+
+ # supply multiple parameters
+ cc_no  => {  
+     constraint  => "cc_number",
+     params      => [ qw( cc_no cc_type ) ],
+ },
+
+ # name a constraint, useful for returning error messages
+ last_name => {
+     name => "ends_in_name",
+     constraint => qr/_name$/,
+ },
+
+Using a hash reference for a constraint permits the passing of multiple
+arguments. Required arguments are C<constraint> or C<constraint_method>.
+Optional arguments are C<name> and C<params>.
+
+A C<name> on a constraints 'glues' the constraint to its error message
+in the validator profile (refer C<msgs> section below). If no C<name> is 
+given then it will default to the value of C<constraint> or 
+C<constraint_method> IF they are NOT a CODE ref or a RegExp ref.
+
+The C<params> value is a reference to an array of the parameters to pass 
+to the constraint method. 
+If an element of the C<params> list is a scalar, it is assumed to be naming
+a key of the %input_hash and that value is passed to the routine. 
+If the parameter is a reference, then it is treated literally and passed 
+unchanged to the routine.
+
+If you are using the older C<constraint> over 
+the new C<constraint_method> then don't forget to include the name of the 
+field to check in the C<params> list. C<constraint_method> provides access
+to this value via the C<get_current_*> methods 
+(refer L<Data::FormValidator::Constraints>)
+
+For more details see L<VALIDATING INPUT BASED ON MULTIPLE FIELDS>.
+
+=head2 constraint_regexp_map (profile key)
+
+This is a supported by deprecated profile key. Using
+C<constraint_methods_regexp_map> is recommended instead.
+
+ constraint_regexp_map => {
+     # All fields that end in _postcode have the 'postcode' constraint applied.
+     qr/_postcode$/    => 'postcode',
+ },                  
+
+A hash ref where the keys are the regular expressions to
+use and the values are the constraints to apply. 
+
+If one or more constraints have already been defined for a given field using
+"constraints", constraint_regexp_map will add an additional constraint for that
+field for each regular expression that matches.
 
 =head1 SEE ALSO
 
@@ -971,10 +1139,9 @@ Validating Web Forms with Perl, L<http://mark.stosberg.com/Tech/perl/form-valida
 
 B<Related modules:>
 
-L<Data::FormValidator::Util::HTML|Data::FormValidator::Util::HTML>
+L<Data::FormValidator::Tutorial|Data::FormValidator::Tutorial> 
 
-L<Data::FormValidator::Tutorial|Data::FormValidator::Tutorial> is now out of date.
-The sample application above may be a more useful introduction. 
+L<Data::FormValidator::Util::HTML|Data::FormValidator::Util::HTML>
 
 L<CGI::Application::ValidateRM|CGI::Application::ValidateRM>, a
 CGI::Application & Data::FormValidator glue module
@@ -1013,11 +1180,29 @@ Albrecht to the MiniVend program.
 
 =head1 BUGS
 
+Bug reports and patches are welcome. Reports which include a failing Test::More
+style test are helpful will receive priority. 
+
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Data-FormValidator> 
 
+=head1 CONTRIBUTING
+
+This project is managed using the darcs source control system (
+http://www.darcs.net/ ). My darcs archive is here:
+http://mark.stosberg.com/darcs_hive/dfv/
+
+B<Support Mailing List>
+ 
+If you have any questions, comments, or feature suggestions, post them to the
+support mailing list!  To join the mailing list, visit
+
+L<http://lists.sourceforge.net/lists/listinfo/cascade-dataform>
+
+Messages about DFV sent directly to the maintainer may be redirected here. 
+
 =head1 AUTHOR
 
-Parts Copyright 2001-2003 by Mark Stosberg <markstos@cpan.org>, (Current Maintainer)
+Parts Copyright 2001-2006 by Mark Stosberg <mark at summersault.com>, (Current Maintainer)
 
 Copyright (c) 1999 Francis J. Lacoste and iNsu Innovations Inc.  All rights reserved.
 (Original Author)
@@ -1026,12 +1211,6 @@ Parts Copyright 1996-1999 by Michael J. Heins <mike@heins.net>
 
 Parts Copyright 1996-1999 by Bruce Albrecht  <bruce.albrecht@seag.fingerhut.com>
 
-B<Support Mailing List>
- 
-If you have any questions, comments, bug reports or feature suggestions,
-post them to the support mailing list!  To join the mailing list, visit 
-L<http://lists.sourceforge.net/lists/listinfo/cascade-dataform>
-
 =head1 LICENSE 
 
 This program is free software; you can redistribute it and/or modify
