@@ -15,12 +15,16 @@ use strict;
 
 package Data::FormValidator::Results;
 
+use Carp;
 use Symbol;
 use Data::FormValidator::Filters qw/:filters/;
 use Data::FormValidator::Constraints (qw/:validators :matchers/);
 use vars qw/$AUTOLOAD $VERSION/;
+use overload
+  'bool' => \&_bool_overload_based_on_success,
+  fallback => 1;
 
-$VERSION = 3.58;
+$VERSION = 4.30;
 
 =pod
 
@@ -42,7 +46,7 @@ Data::FormValidator::Results - results of form input validation.
     # Print the name of invalid fields
     if ( $results->has_invalid ) {
 	foreach my $f ( $results->invalid ) {
-	    print $f, " is invalid: ", $results->invalid( $f ) \n";
+	    print $f, " is invalid: ", $results->invalid( $f ), "\n";
 	}
     }
 
@@ -55,7 +59,7 @@ Data::FormValidator::Results - results of form input validation.
 
     # Print valid fields
     foreach my $f ( $results->valid() ) {
-	print $f, " =  ", $result->valid( $f ), "\n";
+        print $f, " =  ", $results->valid( $f ), "\n";
     }
 
 =head1 DESCRIPTION
@@ -82,7 +86,7 @@ sub _process {
 
  	# Copy data and assumes that all is valid to start with
  		
-	my %data        = $self->_get_data($data);
+	my %data        = $self->_get_input_as_hash($data);
     my %valid	    = %data;
     my @missings    = ();
     my @unknown	    = ();
@@ -95,6 +99,7 @@ sub _process {
     # import valid_* subs from requested packages
 	foreach my $package (_arrayify($profile->{validator_packages})) {
 		if ( !exists $imported_validators{$package} ) {
+			local $SIG{__DIE__}  = \&confess;
 			eval "require $package";
 			if ($@) {
 				die "Couldn't load validator package '$package': $@";
@@ -195,7 +200,7 @@ sub _process {
 		if (ref $valid{$field}) {
 			if ( ref $valid{$field} eq 'ARRAY' ) {
 				for (my $i = 0; $i < scalar @{ $valid{$field} }; $i++) {
-					$valid{$field}->[$i] = undef unless (defined $valid{$field}->[$i] and length $valid{$field}->[$i]);
+					$valid{$field}->[$i] = undef unless (defined $valid{$field}->[$i] and length $valid{$field}->[$i] and $valid{$field}->[$i] !~ /^\x00$/);
 			    }	
                 # If all fields are empty, we delete it.
                 delete $valid{$field} unless grep { defined $_ } @{$valid{$field}};
@@ -203,16 +208,26 @@ sub _process {
 			}
 		}
 		else {
-			delete $valid{$field} unless (defined $valid{$field} and length $valid{$field});
+			delete $valid{$field} unless (defined $valid{$field} and length $valid{$field} and $valid{$field} !~ /^\x00$/);
 		}
 	}
 
     # Check if the presence of some fields makes other optional fields required.
     while ( my ( $field, $deps) = each %{$profile->{dependencies}} ) {
-        if ($valid{$field}) {
+        if (defined $valid{$field}) {
 			if (ref($deps) eq 'HASH') {
 				foreach my $key (keys %$deps) {
-					if($valid{$field} eq $key){
+                    # Handle case of a key with a single value given as an arrayref
+                    # There is probably a better, more general soution to this problem.
+                    my $val_to_compare;
+                    if ((ref $valid{$field} eq 'ARRAY') and (scalar @{ $valid{$field} } == 1)) {
+                        $val_to_compare = $valid{$field}->[0];
+                    }
+                    else {
+                        $val_to_compare = $valid{$field}
+                    }
+
+					if($val_to_compare eq $key){
 						foreach my $dep (_arrayify($deps->{$key})){
 							$required{$dep} = 1;
 						}
@@ -248,8 +263,14 @@ sub _process {
 	}
 
     # Fill defaults
-	while ( my ($field,$value) = each %{$profile->{defaults}} ) {
-		$valid{$field} = $value unless exists $valid{$field};
+    while ( my ($field,$value) = each %{$profile->{defaults}} ) {
+        unless(exists $valid{$field}) {
+            if (ref($value) && ref($value) eq "CODE") {
+                $valid{$field} = $value->($self);
+            } else {
+                $valid{$field} = $value;
+            }
+        }
 	}
 
     # Check for required fields
@@ -269,107 +290,62 @@ sub _process {
 		push @missings, $field unless ($enough_required_fields >= $num_fields_to_require);
 	}
 
-    # add in the constraints from the regexp map 
-	foreach my $re (keys %{ $profile->{constraint_regexp_map} }) {
-		my $sub = _create_sub_from_RE($re);
-
-		# find all the keys that match this RE and add a constraint for them
-		for my $key (keys %valid) {
-			if ($sub->($key)) {
-					my $cur = $profile->{constraints}{$key};
-					my $new = $profile->{constraint_regexp_map}{$re};
-					# If they already have an arrayref of constraints, add to the list
-					if (ref $cur eq 'ARRAY') {
-						push @{ $profile->{constraints}{$key} }, $new;
-					} 
-					# If they have a single constraint defined, create an array ref with with this plus the new one
-					elsif ($cur) {
-						$profile->{constraints}{$key} = [$cur,$new];
-					}
-					# otherwise, a new constraint is created with this as the single constraint
-					else {
-						$profile->{constraints}{$key} = $new;
-					}
-
-					warn "constraint_regexp_map: $key matches\n" if $profile->{debug};
-						
-				}
-			}
-	}
- 
-    # Check constraints
+    # add in the constraints from the regexp maps
+    # We don't want to modify the profile, so we use a new variable.
+	$profile->{constraints} ||= {};
+	my $private_constraints = {
+	    %{ $profile->{constraints} }, 	
+		_add_constraints_from_map($profile,'constraint',\%valid),
+	};
+	$profile->{constraint_methods} ||= {};
+	my $private_constraint_methods = {
+	    %{ $profile->{constraint_methods} }, 	
+		_add_constraints_from_map($profile,'constraint_method',\%valid),
+	};
 
     #Decide which fields to untaint
     my ($untaint_all, %untaint_hash);
-	if (defined($profile->{untaint_constraint_fields})) {
-		if (ref $profile->{untaint_constraint_fields} eq "ARRAY") {
-			foreach my $field (@{$profile->{untaint_constraint_fields}}) {
-				$untaint_hash{$field} = 1;
-			}
-		}
-		elsif ($valid{$profile->{untaint_constraint_fields}}) {
-			$untaint_hash{$profile->{untaint_constraint_fields}} = 1;
-		}
-	}
+    if (defined $profile->{untaint_regexp_map} or defined $profile->{untaint_constraint_fields} ) {
+        # first deal with untaint_constraint_fields
+        if (defined($profile->{untaint_constraint_fields})) {
+            if (ref $profile->{untaint_constraint_fields} eq "ARRAY") {
+                foreach my $field (@{$profile->{untaint_constraint_fields}}) {
+                    $untaint_hash{$field} = 1;
+                }
+            }
+            elsif ($valid{$profile->{untaint_constraint_fields}}) {
+                $untaint_hash{$profile->{untaint_constraint_fields}} = 1;
+            }
+        }
+
+        # now look at untaint_regexp_map
+        if(defined($profile->{untaint_regexp_map})) {
+            my @untaint_regexes;
+            if(ref $profile->{untaint_regexp_map} eq "ARRAY") {
+                @untaint_regexes = @{$profile->{untaint_regexp_map}};
+            }
+            else {
+                push(@untaint_regexes, $profile->{untaint_regexp_map});
+            }
+
+            foreach my $regex (@untaint_regexes) {
+                # look at both constraints and constraint_methods
+                foreach my $field (keys %$private_constraints, keys %$private_constraint_methods) {
+                    next if($untaint_hash{$field}); 
+                    $untaint_hash{$field} = 1 if( $field =~ $regex );
+                }
+            }
+        }
+    }
     elsif ((defined($profile->{untaint_all_constraints}))
 	   && ($profile->{untaint_all_constraints} == 1)) {
 	   $untaint_all = 1;
     }
-    
-	while ( my ($field,$constraint_list) = each %{$profile->{constraints}} ) {
 
-		next unless exists $valid{$field};
+	$self->_check_constraints($private_constraints,\%valid,$untaint_all,\%untaint_hash);
 
-		my $is_constraint_list = 1 if (ref $constraint_list eq 'ARRAY');
-		my $untaint_this =  ($untaint_all || $untaint_hash{$field} || 0);
-
-		my @invalid_list;
-		foreach my $constraint_spec (_arrayify($constraint_list)) {
-			# set current constraint field for use by get_current_constraint_field
-			$self->{__CURRENT_CONSTRAINT_FIELD} = $field;
-
-			my $c = $self->_constraint_hash_build($field,$constraint_spec,$untaint_this);
-
-			my $is_value_list = 1 if (ref $valid{$field} eq 'ARRAY');
-			if ($is_value_list) {
-				foreach (my $i = 0; $i < scalar @{ $valid{$field}} ; $i++) {
-					my @params = $self->_constraint_input_build($c,$valid{$field}->[$i],\%valid);
-
-					# set current constraint field for use by get_current_constraint_value
-					$self->{__CURRENT_CONSTRAINT_VALUE} = $valid{$field}->[$i];
-
-					my ($match,$failed) = _constraint_check_match($c,\@params,$untaint_this);
-					if ($failed) {
-						push @invalid_list, $failed;
-					}
-					else {
-						 $valid{$field}->[$i] = $match if $untaint_this;
-					}
-				}
-			}
-			else {
-				my @params = $self->_constraint_input_build($c,$valid{$field},\%data);
-
-				# set current constraint field for use by get_current_constraint_value
-				$self->{__CURRENT_CONSTRAINT_VALUE} = $valid{$field};
-
-				my ($match,$failed) = _constraint_check_match($c,\@params,$untaint_this);
-				if ($failed) {
-					push @invalid_list, $failed
-				}
-				else {
-					$valid{$field} = $match if $untaint_this;
-				}
-			}
-	   }
-
-		if (@invalid_list) {
-			my @failed = map { $_->{name} } @invalid_list;
-			push @{ $self->{invalid}{$field}  }, @failed;
-            # the older interface to validate returned things differently
-			push @{ $self->{validate_invalid} }, $is_constraint_list ? [$field, @failed] : $field;
-		}
-	}
+	my $force_method_p = 1;
+	$self->_check_constraints($private_constraint_methods,\%valid,$untaint_all,\%untaint_hash, $force_method_p);
 
     # all invalid fields are removed from valid hash
 	foreach my $field (keys %{ $self->{invalid} }) {
@@ -394,6 +370,24 @@ sub _process {
 
 =pod
 
+=head1  success();
+
+This method returns true if there were no invalid or missing fields,
+else it returns false.
+
+As a shorthand, When the $results object is used in boolean context, it is overloaded
+to use the value of success() instead. That allows creation of a syntax like this one used
+in C<CGI::Application::Plugin::ValidateRM>:
+
+ my $results = $self->check_rm('form_display','_form_profile') || return $self->dfv_error_page;
+
+=cut
+
+sub success {
+    my $self = shift;
+    return !($self->has_invalid || $self->has_missing);
+}
+
 =head1  valid( [[field] [, value]] );
 
 In an array context with no arguments, it returns the list of fields which 
@@ -418,7 +412,7 @@ as an array:
  @values = $r->valid('field');
 
 If called with two arguments, it sets C<field> to C<value> and returns C<value>.
-This form is useful to alter the results from within a C<constraint_method>.
+This form is useful to alter the results from within some constraints.
 See the L<Data::FormValidator::Constraints> documentation.
 
  $new_value = $r->valid('field',$new_value);
@@ -444,7 +438,7 @@ sub valid {
 
 =head1 has_missing()
 
-This method returns true if the results contains missing fields.
+This method returns true if the results contain missing fields.
 
 =cut
 
@@ -475,7 +469,7 @@ sub missing {
 
 =head1 has_invalid()
 
-This method returns true if the results contains fields with invalid
+This method returns true if the results contain fields with invalid
 data.
 
 =cut
@@ -510,7 +504,7 @@ sub invalid {
 
 =head1 has_unknown()
 
-This method returns true if the results contains unknown fields.
+This method returns true if the results contain unknown fields.
 
 =cut
 
@@ -548,11 +542,17 @@ This method returns a hash reference to error messages. The exact format
 is determined by parameters in the C<msgs> area of the validation profile,
 described in the L<Data::FormValidator> documentation.
 
-This method takes one possible parameter, a hash reference containing the same 
-options that you can define in the validation profile. This allows you to seperate
-the controls for message display from the rest of the profile. While validation profiles
-may be different for every form, you may wish to format messages the same way
-across many projects.
+B<NOTE:> the C<msgs> parameter in the profile can take a code reference as a
+value, allowing complete control of how messages are generated. If such a code
+reference was provided there, it will be called here instead of the usual
+processing, described below. It will receive as arguments the L<Data::FormValidator::Results>
+object and a hash reference of control parameters.
+
+The hashref passed in should contain the same options that you can define in
+the validation profile. This allows you to separate the controls for message
+display from the rest of the profile. While validation profiles may be
+different for every form, you may wish to format messages the same way across
+many projects.
 
 Controls passed into the <msgs> method will be applied first, followed by ones
 applied in the profile. This allows you to keep the controls you pass to
@@ -561,6 +561,17 @@ C<msgs> as "global" and override them in a specific profile if needed.
 =cut
 
 sub msgs {
+  my $self = shift;
+  my $msgs = $self->{profile}{msgs} || {};
+  if ((ref $msgs eq 'CODE')) {
+    return $msgs->($self,@_);
+  } else {
+    return $self->_generate_msgs(@_);
+  }
+}
+
+
+sub _generate_msgs {
 	my $self = shift;
 	my $controls = shift || {};
 	if (defined $controls and ref $controls ne 'HASH') {
@@ -570,18 +581,31 @@ sub msgs {
 
 	# Allow msgs to be called more than one to accumulate error messages
 	$self->{msgs} ||= {};
-	$self->{profile}->{msgs} ||= {};
+	$self->{profile}{msgs} ||= {};
 	$self->{msgs} = { %{ $self->{msgs} }, %$controls };
 
+    # Legacy typo support. 
+    for my $href ($self->{msgs}, $self->{profile}{msgs}) {
+        if (
+             (not defined $href->{invalid_separator}) 
+             &&  (defined $href->{invalid_seperator})
+         ) {
+            $href->{invalid_separator} = $href->{invalid_seperator};
+        }
+    }
+
 	my %profile = (
 		prefix	=> '',
 		missing => 'Missing',
 		invalid	=> 'Invalid',
-		invalid_seperator => ' ',
+		invalid_separator => ' ',
+
 		format  => '<span style="color:red;font-weight:bold"><span class="dfv_errors">* %s</span></span>',
 		%{ $self->{msgs} },
-		%{ $self->{profile}->{msgs} },
+		%{ $self->{profile}{msgs} },
 	);
+
+
 	my %msgs = ();
 
 	# Add invalid messages to hash
@@ -590,7 +614,7 @@ sub msgs {
 	if ($self->has_invalid) {
 		my $invalid = $self->invalid;
 		for my $i ( keys %$invalid ) {
-			$msgs{$i} = join $profile{invalid_seperator}, map {
+			$msgs{$i} = join $profile{invalid_separator}, map {
 				_error_msg_fmt($profile{format},($profile{constraints}{$_} || $profile{invalid}))
 				} @{ $invalid->{$i} };
 		}
@@ -606,7 +630,9 @@ sub msgs {
 
 	my $msgs_ref = prefix_hash($profile{prefix},\%msgs);
 
-	$msgs_ref->{ $profile{any_errors} } = 1 if defined $profile{any_errors};
+    if (! $self->success) {
+    	$msgs_ref->{ $profile{any_errors} } = 1 if defined $profile{any_errors};
+    }
 
 	return $msgs_ref;
 
@@ -639,8 +665,8 @@ writing your own complex constraint.
 		height => '60',
 	});
 
-This function does not currently multi-valued fields. If it 
-does in the future, the above syntax will still work..
+This function does not currently support multi-valued fields. If it 
+does in the future, the above syntax will still work.
 
 =cut
 
@@ -674,7 +700,14 @@ sub meta {
 
 sub get_input_data {
 	my $self = shift;
-	return $self->{__INPUT_DATA};
+    my %p = @_;
+    if ($p{as_hashref}) {
+        my %hash = $self->_get_input_as_hash( $self->{__INPUT_DATA} );
+        return \%hash;
+    }
+    else {
+	    return $self->{__INPUT_DATA};
+    }
 }
 
 sub get_current_constraint_field {
@@ -692,6 +725,25 @@ sub get_current_constraint_name {
 	return $self->{__CURRENT_CONSTRAINT_NAME};
 }
 
+sub untainted_constraint_value {
+    my $self = shift;
+    my $match = shift;
+
+    return undef unless defined $match;
+    return $self->{__UNTAINT_THIS} ? $match : length $match;
+}
+
+sub set_current_constraint_name {
+	my $self = shift;
+	my $value = shift;
+	$self->{__CURRENT_CONSTRAINT_NAME} = $value;
+} 
+# same as above
+sub name_this {
+	my $self = shift;
+	my $value = shift;
+	$self->{__CURRENT_CONSTRAINT_NAME} = $value;
+} 
 
 # INPUT: prefix_string, hash reference
 # Copies the hash and prefixes all keys with prefix_string
@@ -709,18 +761,20 @@ sub prefix_hash {
 
 
 # We tolerate two kinds of regular expression formats
-# First, the preferred format made with "qr", matched using a learning paren
+# First, the preferred format made with "qr", matched using a leading paren
 # Also, we accept the deprecated format given as strings: 'm/old/'
 # (which must start with a slash or "m", not a paren)
 sub _create_sub_from_RE {
 	my $re = shift || return undef;
 	my $untaint_this = shift;
+    my $force_method_p = shift;
 
 	my $sub;
 	# If it's "qr" style
 	if (substr($re,0,1) eq '(') {
 		$sub = sub { 
-            my $val = shift;
+            # With methods, the value is the second argument
+            my $val = $force_method_p ? $_[1] : $_[0];
 			my ($match) = ($val =~ $re); 
 			if ($untaint_this && defined $match) {
                 # pass the value through a RE that matches anything to untaint it.
@@ -734,15 +788,22 @@ sub _create_sub_from_RE {
 
 	}
 	else {
+        local $SIG{__DIE__}  = \&confess;
         my $return_code = ($untaint_this) ? '; return ($& =~ m/(.*)/s)[0] if defined($`);' : '';
-		$sub = eval 'sub { $_[0] =~ '.$re.$return_code. '}';
+        # With methods, the value is the second argument
+        if ($force_method_p) {
+            $sub = eval 'sub { $_[1] =~ '.$re.$return_code. '}';
+        }
+        else {
+            $sub = eval 'sub { $_[0] =~ '.$re.$return_code. '}';
+        }
 	    die "Error compiling regular expression $re: $@" if $@;
 	}
 	return $sub;
 }
 
 
-sub _error_msg_fmt ($$) {
+sub _error_msg_fmt  {
 	my ($fmt,$msg) = @_;
 	$fmt ||= 
 			'<span style="color:red;font-weight:bold"><span class="dfv_errors">* %s</span></span>';
@@ -785,16 +846,32 @@ sub _filter_apply {
 	}
 }
 
+# =head2 _constraint_hash_build()
+# 
+# $constraint_href = $self->_constraint_hash_build($spec,$untaint_p)
+#
+# Input:
+#   - $spec           # Any constraint valid in the profile
+#   - $untaint        # bool for whether we could try to untaint the field. 
+#   - $force_method_p # bool for if it's  a method ?
+#
+# Output:
+#  - $constraint_hashref
+#    Keys are as follows:  
+# 		constraint - the constraint as coderef
+# 		name	   - the constraint name, if we know it. 
+# 		params	   - 'params', as given in the hashref style of specifying a constraint
+# 		is_method  - bool for whether this was a 'constraint' or 'constraint_method'
+
 sub _constraint_hash_build {
-	my ($self,$field,$constraint_spec,$untaint_this) = @_;
-	die "_constraint_apply received wrong number of arguments" unless (scalar @_ == 4);
+	my ($self,$constraint_spec,$untaint_this,$force_method_p) = @_;
+	die "_constraint_hash_build received wrong number of arguments" unless (scalar @_ == 4);
 
 	my	$c = {
 			name 		=> $constraint_spec,
 			constraint  => $constraint_spec, 
 		};
 
-
    # constraints can be passed in directly via hash
 	if (ref $c->{constraint} eq 'HASH') {
 			$c->{constraint} = ($constraint_spec->{constraint_method} || $constraint_spec->{constraint});
@@ -806,7 +883,7 @@ sub _constraint_hash_build {
 	# Check for regexp constraint
 	if ((ref $c->{constraint} eq 'Regexp')
 			or ( $c->{constraint} =~ m@^\s*(/.+/|m(.).+\2)[cgimosx]*\s*$@ )) {
-		$c->{constraint} = _create_sub_from_RE($c->{constraint},$untaint_this);
+		$c->{constraint} = _create_sub_from_RE($c->{constraint},$untaint_this,$force_method_p);
 	}
 	# check for code ref
 	elsif (ref $c->{constraint} eq 'CODE') {
@@ -816,9 +893,6 @@ sub _constraint_hash_build {
 		# provide a default name for the constraint if we don't have one already
 		$c->{name} ||= $c->{constraint};
 
-		# Save the current constraint name for later
-		$self->{__CURRENT_CONSTRAINT_NAME} = $c->{name};
-		
 		#If untaint is turned on call match_* sub directly. 
 		if ($untaint_this) {
 			my $routine = 'match_'.$c->{constraint};			
@@ -828,6 +902,7 @@ sub _constraint_hash_build {
 			}
 			# If the constraint name starts with RE_, try looking for it in the Regexp::Common package
 			elsif ($c->{constraint} =~ m/^RE_/) {
+				local $SIG{__DIE__}  = \&confess;
 				$c->{is_method} = 1;
 				$c->{constraint} = eval 'sub { &_create_regexp_common_constraint(@_)}' 
 					|| die "could not create Regexp::Common constraint: $@";
@@ -839,6 +914,7 @@ sub _constraint_hash_build {
 			# try to use match_* first
 			my $routine = 'match_'.$c->{constraint};			
 			if (defined *{qualify_to_ref($routine)}{CODE}) {
+				local $SIG{__DIE__}  = \&confess;
 				$c->{constraint} = eval 'sub { no strict qw/refs/; return defined &{"match_'.$c->{constraint}.'"}(@_)}';
 			}
 			# match_* doesn't exist; if it is supposed to be from the
@@ -848,6 +924,7 @@ sub _constraint_hash_build {
 			}
 			# Load it from Regexp::Common 
 			elsif ($c->{constraint} =~ m/^RE_/) {
+				local $SIG{__DIE__}  = \&confess;
 				$c->{is_method} = 1;
 				$c->{constraint} = eval 'sub { return defined &_create_regexp_common_constraint(@_)}' ||
 					die "could not create Regexp::Common constraint: $@";
@@ -858,10 +935,21 @@ sub _constraint_hash_build {
 		}
 	}
 
+	# Save the current constraint name for later
+	$self->{__CURRENT_CONSTRAINT_NAME} = $c->{name};
+
 	return $c;
 
 }
 
+# =head2 _constraint_input_build()
+# 
+#  @params = $self->constraint_input_build($c,$value,$data);
+# 
+# Build in the input that passed into the constraint. 
+# 
+# =cut
+
 sub _constraint_input_build {
 	my ($self,$c,$value,$data) = @_;
 	die "_constraint_input_build received wrong number of arguments" unless (scalar @_ == 4);
@@ -881,45 +969,60 @@ sub _constraint_input_build {
 	return @params;
 }
 
+# =head2 _constraint_check_match()
+#
+# ($value,$failed_href) = $self->_constraint_check_match($c,\@params,$untaint_this);
+#
+# This is the routine that actually, finally, checks if a constraint passes or fails.
+#
+# Input:
+#   - $c,            a constraint hash, as returned by C<_constraint_hash_build()>.
+#   - \@params,      params to pass to the constraint, as prepared by C<_constraint_input_build()>. 
+#   - $untaint_this  bool if we untaint successful constraints. 
+#
+# Output:
+#  - $value          the value if successful
+#  - $failed_href    a hashref with the following keys:
+#		- failed     bool for failure or not
+#	    - name	     name of the failed constraint, if known. 
+
 sub _constraint_check_match {
-	my 	($c,$params,$untaint_this) = @_;
-	die "_constraint_check_match received wrong number of arguments" unless (scalar @_ == 3);
+	my 	($self,$c,$params,$untaint_this) = @_;
+	die "_constraint_check_match received wrong number of arguments" unless (scalar @_ == 4);
+
+    # Store whether or not we want untainting in the object so that constraints
+    # can do the right thing conditionally.
+    $self->{__UNTAINT_THIS} = $untaint_this;
 
     my $match = $c->{constraint}->( @$params );
 
     # We need to make this distinction when untainting,
     # to allow untainting values that are defined but not true,
     # such as zero.
-    my $success =  defined $match;
+    my $success;
     if (defined $match) {
        $success =  ($untaint_this) ? length $match : $match;
     }
-    
-	if ($success) { 
-		return $match;
-	}
-	else {
-		return 
-		undef,	
+
+	my $failed = 1 unless $success;
+	return (
+		$match,
 		{
-			failed  => 1,
-			name	=> $c->{name},
-		};
-	}
+			failed  => $failed,
+			name	=> $self->{__CURRENT_CONSTRAINT_NAME},
+		},
+	);
 }
 
 # Figure out whether the data is a hash reference of a param-capable object and return it has a hash
-sub _get_data {
+sub _get_input_as_hash {
 	my ($self,$data) = @_;
 	$self->{__INPUT_DATA} = $data;
 	require UNIVERSAL;
 
-    # This checks whether we have an object or not.
-    if (UNIVERSAL::isa($data,'UNIVERSAL') ) {
+    # This checks whether we have an object that supports param
+    if (UNIVERSAL::can($data,'param') ) {
 		my %return;
-		# make sure object supports param()
-		defined($data->UNIVERSAL::can('param')) or
-		die "Data::FormValidator->validate() or check() called with an object which lacks a param() method!";
 		foreach my $k ($data->param()){
 			# we expect param to return an array if there are multiple values
 			my @v = $data->param($k);
@@ -928,15 +1031,41 @@ sub _get_data {
 		return %return;
 	}
 	# otherwise, it's already a hash reference
+    elsif (ref $data eq 'HASH') {
+        # be careful to actually copy array references
+        my %copy = %$data;
+        for (grep { ref $data->{$_} eq 'ARRAY' } keys %$data) {
+            my @array_copy = @{ $data->{$_} };
+            $copy{$_} = \@array_copy;
+        }
+
+		return %copy;
+    }
 	else {
-		return %$data;	
+		die "Data::FormValidator->validate() or check() called with invalid input data structure.";
 	}
 }
 
+# A newer version of this logic now exists in Constraints.pm in the AUTOLOADing section
+# This is is used to support the older param passing style. Eg:
+#
+# {
+#	constraint => 'RE_foo_bar',
+#	params => [ \'zoo' ]
+#  }
+#
+# Still, it's possble, the two bits of logic could be refactored into one location if you cared
+# to do that. 
 
 sub _create_regexp_common_constraint  {
+	# this should work most of the time and is useful for preventing warnings
+
+	# prevent name space clashes
+    package Data::FormValidator::Constraints::RegexpCommon;		
+	
 	require Regexp::Common;
 	import  Regexp::Common 'RE_ALL';
+
 	my $self = shift;
 	my $re_name = $self->get_current_constraint_name;
 	# deference all input
@@ -947,6 +1076,143 @@ sub _create_regexp_common_constraint  {
 	return ($self->get_current_constraint_value =~ qr/^$re$/) ? $1 : undef; 
 }
 
+# _add_constraints_from_map($profile,'constraint',\%valid);
+# Returns:
+#  - a hash to add to either 'constraints' or 'constraint_methods'
+
+sub _add_constraints_from_map {
+	die "_add_constraints_from_map: need 3 arguments" unless (scalar @_ == 3);
+	my ($profile, $name, $valid) = @_;
+	($name =~ m/^constraint(_method)?$/) || die "unexpected input.";
+
+	my $key_name = $name.'s';
+	my $map_name = $name.'_regexp_map';
+
+	my %result = ();
+	foreach my $re (keys %{ $profile->{$map_name} }) {
+		my $sub = _create_sub_from_RE($re);
+
+		# find all the keys that match this RE and add a constraint for them
+		for my $key (keys %$valid) {
+			if ($sub->($key)) {
+					my $cur = $profile->{$key_name}{$key};
+					my $new = $profile->{$map_name}{$re};
+					# If they already have an arrayref of constraints, add to the list
+					if (ref $cur eq 'ARRAY') {
+						push @{ $result{$key} }, $new;
+					} 
+					# If they have a single constraint defined, create an array ref with with this plus the new one
+					elsif ($cur) {
+						$result{$key} = [$cur,$new];
+					}
+					# otherwise, a new constraint is created with this as the single constraint
+					else {
+						$result{$key} = $new;
+					}
+					warn "$map_name: $key matches\n" if $profile->{debug};
+				}
+			}
+	}
+	return %result;
+}
+
+sub _bool_overload_based_on_success {
+    my $results = shift;
+    return $results->success()
+}
+
+# =head2 _check_constraints()
+#
+# $self->_check_constraints(
+#	$profile->{constraint_methods},
+#	\%valid, 
+#	$untaint_all
+#	\%untaint_hash
+#	$force_method_p
+#);
+#
+# Input:
+#  - 'constraints' or 'constraint_methods' hashref
+#  - hashref of valid data
+#  - bool to try to untaint everything
+#  - hashref of things to untaint
+#  - bool if all constraints should be treated as methods. 
+	
+sub _check_constraints {
+	my ($self, 
+	    $constraint_href, 
+		$valid, 
+		$untaint_all,
+		$untaint_href,
+		$force_method_p) = @_; 
+
+	while ( my ($field,$constraint_list) = each %$constraint_href ) {
+		next unless exists $valid->{$field};
+
+		my $is_constraint_list = 1 if (ref $constraint_list eq 'ARRAY');
+		my $untaint_this = ($untaint_all || $untaint_href->{$field} || 0);
+
+		my @invalid_list;
+        # used to insure we only bother recording each failed constraint once
+		my %constraints_seen;
+		foreach my $constraint_spec (_arrayify($constraint_list)) {
+
+			# set current constraint field for use by get_current_constraint_field
+			$self->{__CURRENT_CONSTRAINT_FIELD} = $field;
+
+			# Initialize the current constraint name to undef, to prevent it
+			# from being accidently shared
+			$self->{__CURRENT_CONSTRAINT_NAME} = undef;
+
+			my $c = $self->_constraint_hash_build($constraint_spec,$untaint_this, $force_method_p);
+			$c->{is_method} = 1 if $force_method_p;
+
+			my $is_value_list = 1 if (ref $valid->{$field} eq 'ARRAY');
+            my %param_data = ( $self->_get_input_as_hash($self->get_input_data) , %$valid );
+			if ($is_value_list) {
+				foreach (my $i = 0; $i < scalar @{ $valid->{$field}} ; $i++) {
+                    if( !exists $constraints_seen{\$c} ) {
+
+                        my @params = $self->_constraint_input_build($c,$valid->{$field}->[$i],\%param_data);
+
+                        # set current constraint field for use by get_current_constraint_value
+                        $self->{__CURRENT_CONSTRAINT_VALUE} = $valid->{$field}->[$i];
+
+                        my ($match,$failed) = $self->_constraint_check_match($c,\@params,$untaint_this);
+                        if ($failed->{failed}) {
+                            push @invalid_list, $failed;
+                            $constraints_seen{\$c} = 1;
+                        }
+                        else {
+                            $valid->{$field}->[$i] = $match if $untaint_this;
+                        }
+                    }
+				}
+			}
+			else {
+				my @params = $self->_constraint_input_build($c,$valid->{$field},\%param_data);
+
+				# set current constraint field for use by get_current_constraint_value
+				$self->{__CURRENT_CONSTRAINT_VALUE} = $valid->{$field};
+
+				my ($match,$failed) = $self->_constraint_check_match($c,\@params,$untaint_this);
+				if ($failed->{failed}) {
+					push @invalid_list, $failed
+				}
+				else {
+					$valid->{$field} = $match if $untaint_this;
+				}
+			}
+	   }
+
+		if (@invalid_list) {
+			my @failed = map { $_->{name} } @invalid_list;
+			push @{ $self->{invalid}{$field}  }, @failed;
+            # the older interface to validate returned things differently
+			push @{ $self->{validate_invalid} }, $is_constraint_list ? [$field, @failed] : $field;
+		}
+	}
+}
 
 1;
 
